###############################################################################
#
# IBT: Isolated Build Tool
# Copyright (C) 2016, Richard Cook. All rights reserved.
#
# Simple wrappers around Docker etc. for fully isolated build environments
#
###############################################################################

__project_name__ = "ibt"
__version__ = "0.5"
__description__ = "Simple wrappers around Docker etc. for fully isolated build environments"
